/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.yezza.controllers;

// the controller uses the DAO and business Layers classes
import com.yezza.business.User;
import com.yezza.dao.UserDAO;   
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.concurrent.atomic.AtomicReference;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.yezza.utils.Message;

/**
 *
 * @author ayezza
 */
public class UserCRUD extends HttpServlet {
    private static final long serialVersionUID = 1L;

    String DATE_FORMAT="yyyy-MM-dd";
    SimpleDateFormat format = new SimpleDateFormat(DATE_FORMAT);
    
    private boolean ValidateDate(String pDate){
        try {
            DateFormat df = new SimpleDateFormat(DATE_FORMAT);
            df.setLenient(false);
            df.parse(pDate);
            return true;
        } catch (ParseException e) {
            return false;
        }
    }
    private boolean ValidatePassword(String pPassword){
        return (pPassword.length()>=4 && pPassword.length()<=32)?true:false;
    }
    private boolean ValidateEmail(String pEmail){
        return pEmail.matches("^[_A-Za-z0-9-\\+]+(\\.[_A-Za-z0-9-]+)*@[A-Za-z0-9-]+(\\.[A-Za-z0-9]+)*(\\.[A-Za-z]{2,})$");
    }
    private boolean ValidateByMinMAx(String str, int minLen, int maxlen){
        return (str.length()>=minLen && str.length()<=maxlen)?true:false;
    }
    private boolean ValidateLastName(String pLastName){
        return ValidateByMinMAx (pLastName,3, 32);
    }
    private boolean ValidateFirstName(String pFirstName){
        return ValidateByMinMAx (pFirstName,3, 32);
    }
    private boolean ValidateUserData(User pUser, AtomicReference<String> errorMsg){
       String retString="";
       boolean b1=ValidateLastName(pUser.getNom());
       if (!b1) retString+="Le nom doit être compris entre 4 et 32 cacactères.";
       boolean b2=ValidateFirstName(pUser.getPrenom());
       if (!b2) retString+="\nLe prénom doit être compris entre 4 et 32 cacactères.";
       boolean b3=ValidateEmail(pUser.getEmail());
       if (!b3) retString+="\nL'adresse email est non valide.";
       boolean b4=ValidatePassword(pUser.getPwd());
       if (!b4) retString+="\nLe mot de passe doit être compris entre 4 et 32 cacactères.";
       String dateString=format.format(pUser.getDateNaissance());
       boolean b5=ValidateDate(dateString);
       if (!b5) retString+="\nLa date est non valide.";
       
       errorMsg.set(retString);
       
       return b1&&b2&&b3&&b4&&b5;
    }
    
    private void getAllUsers(HttpServletResponse response, HttpServletRequest request){
        
        try {
            UserDAO userDAO=new UserDAO();
            String gsonString=userDAO.getAllUsers();
            response.setCharacterEncoding("UTF-8");
            
            response.getWriter().write(gsonString);
        } catch (IOException ex) {
            Logger.getLogger(UserCRUD.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    
    private void addUser(HttpServletRequest request) throws UnsupportedEncodingException{
        //To pass errorMsg by reference to the called function
        AtomicReference<String> errorMsg = new  AtomicReference<>("");

        try {
            User pUser=new User( request.getParameter("nom"), request.getParameter("prenom"),
                    request.getParameter("email"), request.getParameter("password"),  
                    format.parse(request.getParameter("dateNaissance")));
            if (!ValidateUserData(pUser, errorMsg) ) {
                request.setAttribute("message", new Message("The user has not been added. Validation error."+ errorMsg.get(), 0, "red")  );
                return;
            }
            
            UserDAO userDAO=new UserDAO();
            
            if (userDAO.addUser(pUser, errorMsg)) {
                request.setAttribute("message", new Message("User added successfully."+ errorMsg.get(), 0, "green") );
            } else request.setAttribute("message", new Message("User add fails. " + errorMsg.get(), 0, "red") );
        } catch (ParseException ex) {
            Logger.getLogger(UserCRUD.class.getName()).log(Level.SEVERE, null, ex);
            errorMsg.set(ex.getLocalizedMessage());
            request.setAttribute("message", new Message("User add fails. " + errorMsg.get(), 0, "red") );
        }
    }
    private void delUser(HttpServletRequest request){
        //To pass errorMsg by reference to the called function
        AtomicReference<String> errorMsg = new  AtomicReference<>("");

            
        try {
            User pUser=new User(Integer.parseInt(request.getParameter("id")), request.getParameter("nom"), request.getParameter("prenom"),
                    request.getParameter("email"), request.getParameter("password"),  format.parse(request.getParameter("dateNaissance")));
            UserDAO userDAO=new UserDAO();
            
            if (userDAO.delUser(pUser, errorMsg) ) {
                request.setAttribute("message", new Message("User deleted successfully." + errorMsg.get(), 0, "green") ) ;
            } else request.setAttribute("message", new Message("User Delete fails. " + errorMsg.get() , 0, "red") ) ;
        } catch (ParseException ex) {
            Logger.getLogger(UserCRUD.class.getName()).log(Level.SEVERE, null, ex);
            errorMsg.set(ex.getLocalizedMessage());
            request.setAttribute("message", new Message("User Delete fails. " + errorMsg.get() , 0, "red") );
        }
    }
    private void updateUser(HttpServletRequest request) throws UnsupportedEncodingException{
        //To pass errorMsg by reference to the called function
        AtomicReference<String> errorMsg = new  AtomicReference<>("");
    
        try {
            User pUser=new User(Integer.parseInt(request.getParameter("id")), request.getParameter("nom"), request.getParameter("prenom"),
                    request.getParameter("email"), request.getParameter("password"),  format.parse(request.getParameter("dateNaissance")));
            UserDAO userDAO=new UserDAO();
            
            if (userDAO.updateUser(pUser, errorMsg)) {
                request.setAttribute("message", new Message("User updated successfully." + errorMsg.get(), 0, "green") );
            } else request.setAttribute("message", new Message("User update fails. " + errorMsg.get() , 0, "red") );
        } catch (ParseException ex) {
            Logger.getLogger(UserCRUD.class.getName()).log(Level.SEVERE, null, ex);
            errorMsg.set(ex.getLocalizedMessage());
            request.setAttribute("message", new Message("User update fails. " + errorMsg.get() , 0, "red") );
        }
    }
    
    private void SearchById(HttpServletRequest request, HttpServletResponse response) {
        try {
            int searchWhat= Integer.parseInt( request.getParameter("searchWhatVal") );
            UserDAO userDAO=new UserDAO();
            String UsersJsonString=userDAO.getUserById(searchWhat);
            response.setCharacterEncoding("UTF-8");
            response.getWriter().write(UsersJsonString);
            
        } catch (IOException ex) {
            Logger.getLogger(UserCRUD.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    private void SearchByMail(HttpServletRequest request, HttpServletResponse response) {
        try {
            String searchWhat=  request.getParameter("searchWhatVal") ;
            UserDAO userDAO=new UserDAO();
            String UsersJsonString=userDAO.getUserByEmail(searchWhat);
            response.setCharacterEncoding("UTF-8");
            response.getWriter().write(UsersJsonString);
            
        } catch (IOException ex) {
            Logger.getLogger(UserCRUD.class.getName()).log(Level.SEVERE, null, ex);
        }
        
    }
    private void SearchByNom(HttpServletRequest request, HttpServletResponse response) {
        try {
            String searchWhat=request.getParameter("searchWhatVal");
            UserDAO userDAO=new UserDAO();
            String UsersJsonString=userDAO.getUsersByName("%"+searchWhat+"%");
            response.setCharacterEncoding("UTF-8");
            response.getWriter().write(UsersJsonString);
            
            request.setAttribute("message", new Message( ("".equals(UsersJsonString) )?"No user found !":"Utiliser les bouton de navigation pour afficher les utilisateurs." , 0, "red") );
        } catch (IOException ex) {
            Logger.getLogger(UserCRUD.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    private void SearchByPrenom(HttpServletRequest request, HttpServletResponse response) {
        try {
            String searchWhat=request.getParameter("searchWhatVal");
            UserDAO userDAO=new UserDAO();
            String UsersJsonString=userDAO.getUsersByPrenom(searchWhat);
            response.setCharacterEncoding("UTF-8");
            response.getWriter().write(UsersJsonString);
            
            request.setAttribute("message", new Message( ("".equals(UsersJsonString))?"No user found !":"Utiliser les bouton de navigation pour afficher les utilisateurs." , 0, "red") ) ;
        } catch (IOException ex) {
            Logger.getLogger(UserCRUD.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    private void fillDownUserFields(HttpServletRequest request, User user, String message) {
        request.setAttribute("message", message);
        request.setAttribute("User", user);
    }
    
    
    /**
     * Handles the HTTP <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        response.addHeader("Content-Type", "text/html; charset=UTF-8");
        request.setCharacterEncoding("UTF-8");
        String submitValue=request.getParameter("crud");
        
        if ("getAll".equals(submitValue)) {
            getAllUsers(response, request);
            return;
        }
        if ("addUser".equals(submitValue)) {
            addUser(request);
        }
        if ("delUser".equals(submitValue)) {
            delUser(request);
        }
        if ("updateUser".equals(submitValue)) {
           updateUser(request);
        }
       
        if ("search".equals(submitValue)) {
            String searchBy=request.getParameter("searchByVal");
            
            if ("email".equals(searchBy)) {
                SearchByMail(request, response);
            }
            if ("id".equals(searchBy)) {
              SearchById(request, response);
            }
            if ("nom".equals(searchBy)) {
              SearchByNom(request, response);
            }
            if ("prenom".equals(searchBy)) {
              SearchByPrenom(request, response);
            }
            return;
        }
        
        request.getRequestDispatcher("/index.jsp").forward(request, response);
    }

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
    }
    
    
    
    
    
}

